"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const koa_1 = __importDefault(require("koa"));
const Application_1 = require("../Application");
const Middleware_1 = require("../Middleware");
class ServerService {
    constructor(mode, port, middleware) {
        this.middleware = [];
        this.port = port;
        if (mode === Application_1.ApplicationModes.Server) {
            this.server = new koa_1.default();
            this.middleware = this.server.middleware;
        }
        this.use(...middleware);
    }
    use(...middlewares) {
        middlewares.forEach(middleware => {
            this.middleware.push(middleware instanceof Middleware_1.AbstractMiddleware ? middleware.asCallback() : middleware);
        });
        return this;
    }
    useBefore(middleware, ...middlewares) {
        return this.updateMiddleware(this.indexOfMiddleware(middleware), 0, ...middlewares);
    }
    useAfter(middleware, ...middlewares) {
        return this.updateMiddleware(this.indexOfMiddleware(middleware) + 1, 0, ...middlewares);
    }
    replace(middleware, ...middlewares) {
        return this.updateMiddleware(this.indexOfMiddleware(middleware), 1, ...middlewares);
    }
    updateMiddleware(at, remove, ...middlewares) {
        this.middleware.splice(at === -1 ? 0 : at, remove, ...middlewares);
        return this;
    }
    indexOfMiddleware(middleware) {
        if (!this.server) {
            return -1;
        }
        return this.middleware.findIndex(suspect => {
            if (typeof middleware === 'string') {
                if (suspect._name) {
                    return suspect._name === middleware;
                }
                return suspect.name === middleware;
            }
            if (suspect._fromClass) {
                return middleware === suspect._fromClass;
            }
            return suspect === middleware;
        });
    }
    getServer() {
        return this.server;
    }
    start() {
        this.server.listen(this.port);
        return this;
    }
}
exports.ServerService = ServerService;
//# sourceMappingURL=ServerService.js.map