"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ServiceManager_1 = require("../../../src/Library/ServiceManager");
describe('ServiceManager', () => {
    class UselessService {
        constructor(value) {
            this.value = value;
        }
    }
    const serviceManagerFactory = () => new ServiceManager_1.ServiceManager({
        services: new Map([[Date, new Date]]),
        invokables: new Map([['StupidWayToGetADateInstance', Date]]),
        factories: new Map([
            ['UselessService', () => new UselessService('Really String')],
            [UselessService, () => new UselessService('Really Reference')],
            [Date, () => 'This should never be called'],
        ]),
        aliases: {
            uselessAlias: UselessService,
        },
    });
    describe('constructor()', () => {
        it('Should construct properly without any arguments.', () => {
            const serviceManager = new ServiceManager_1.ServiceManager();
            expect(serviceManager).toBeInstanceOf(ServiceManager_1.ServiceManager);
        });
        it('Should construct and register services when provided with a config.', () => {
            const serviceManager = serviceManagerFactory();
            expect(serviceManager['services'].get(Date)).toBeInstanceOf(Date);
            expect(serviceManager['factories'].get('UselessService')()).toBeInstanceOf(UselessService);
            expect(serviceManager['aliases'].uselessAlias).toBe(UselessService);
            expect(typeof serviceManager['factories'].get('StupidWayToGetADateInstance')).toBe('function');
            expect(serviceManager['factories'].get('StupidWayToGetADateInstance')()).toBeInstanceOf(Date);
        });
    });
    describe('.get()', () => {
        const serviceManager = serviceManagerFactory();
        it('resolves to a service based on an alias key', () => {
            expect(serviceManager.get('uselessAlias')).toStrictEqual(serviceManager.get(UselessService));
        });
        it('should not call the factory if the service already exists', () => {
            expect(serviceManager.get(Date)).toBeInstanceOf(Date);
        });
        it('should register the service after calling the factory', () => {
            expect(serviceManager['services'].has('UselessService')).toBe(false);
            expect(serviceManager.get('UselessService')).toBeInstanceOf(UselessService);
            expect(serviceManager['services'].has('UselessService')).toBe(true);
        });
    });
});
//# sourceMappingURL=ServiceManager.test.js.map